#!/bin/bash


MODNAME="$1"
DEFOPTS="$2"
KVER="$3"

test -z "$MODNAME" && MODNAME="$(basename *.ko .ko)"

test "$MODNAME" = "*" &&
{
    echo "No .ko file found in current directory to install"
    exit 1
}

test -f $MODNAME.ko ||
{
    echo "Module $MODNAME.ko not found in current directory"
    exit 1
}

test -z "$KVER" && KVER="$(uname -r)"

test -n "$KVER" ||
{
    echo "Can't determine kernel vesion for install."
    exit 1
}

echo "Installing $MODNAME moudle for kernel version $KVER"
echo

test -d /lib/modules/$KVER/kernel ||
{
    echo "Directory /lib/modules/$KVER/kernel not found."
    echo "Don't know where to install $MODNAME.ko"
    exit 1
}

mkdir -p /lib/modules/$KVER/kernel/drivers

# remove old-style config file and init script
INITD=$(find /etc/ -type d -name "init.d")
if [ "$MODNAME" = rp2 ]; then
    rm -f /etc/rp_infinity_express.conf
    test -n "$INITD" && rm -f $(find $INITD -name rp_infinity_express)
elif [ "$MODNAME" = rocket ]; then
    rm -f $(find /etc/ -type l -name "*rocket")
    rm -f /sbin/loadrm2 /etc/ctmmdmfw.rm /etc/ctmmdmld.rm
    test -n "$INITD" && rm -f $(find $INITD -name rocket)
fi

# install kernel module
#
# if there is an $MODNAME.ko installed by the distro, it should be in
# kernel/drivers/tty/serial, so we'll install ours above that to
# make sure modprobe finds it first.  Just to be safe, we'll also
# remove any existing $MODNAME driver module.

rm -f $(find /lib/modules/$KVER/kernel/drivers -name $MODNAME.ko)
mkdir -p /lib/modules/$KVER/kernel/drivers/tty
echo "installing $MODNAME kernel module in /lib/modules/$KVER/kernel/drivers/tty"
cp $MODNAME.ko /lib/modules/$KVER/kernel/drivers/tty
echo "updating module dependancies"
/sbin/depmod $KVER || depmod $KVER
echo "done"
echo

echo "Setting up '$MODNAME' module:"
echo

# make sure module gets loaded -- not needed for newer systems that
# will recognize the PCI ID and automatically load the kernel module,
# but wonn't hurt anything on those newer systems

if [ -d /etc/modules-load.d ]; then
    if test -f /etc/modules-load.d/$MODNAME.conf; then
        echo "/etc/modules-load.d/$MODNAME.conf already exists"
    else
        echo "creating /etc/modules-load.d/$MODNAME.conf"
        echo "$MODNAME" > /etc/modules-load.d/$MODNAME.conf
    fi
elif [ -f /etc/conf.d/modules ]; then
    if grep -q "^modules=.* $MODNAME" /etc/conf.d/modules; then
        echo "/etc/conf.d/modules already contains $MODNAME"
    else
        echo "appending $MODNAME to /etc/conf.d/modules"
        echo "modules=\"\${modules} $MODNAME\"" >> /etc/conf.d/modules
    fi
elif [ -f /etc/sysconfig/modules ]; then
    if grep -q "modprobe $MODNAME" /etc/sysconfig/modules; then
        echo "/etc/sysconfig/modules already contains $MODNAME"
    else
        echo "appending $MODNAME to /etc/sysconfig/modules"
        echo 'modprobe $MODNAME' >> /etc/sysconfig/modules
    fi
elif [ -f /etc/modules ]; then
    if grep -q "$MODNAME" /etc/modules; then
        echo "/etc/modules already contains $MODNAME"
    else
        echo "appending $MODNAME to /etc/modules"
        echo "$MODNAME" >> /etc/modules
    fi
elif [ -f /etc/rc.modules ]; then
    if grep -q "modprobe $MODNAME" /etc/rc.modules; then
        echo "/etc/rc.modules already contains $MODNAME"
    else
        echo "appending $MODNAME to /etc/rc.modules"
        echo 'modprobe $MODNAME' >> /etc/rc.modules
    fi
else
    if fgrep -q /etc/rc.modules /etc/rc.sysinit 2>/dev/null; then
        echo "found /etc/rc.modules referenced in rc.sysinit: creating /etc/rc.modules"
    else
        echo "\

Didn't find any of known mechanisms used to load modules at startup:

   /etc/modules-load.d/
   /etc/conf.d/modules
   /etc/modules
   /etc/rc.modules

Creating /etc/rc.modules file.  If $MODNAME module does not get loaded
at loaded at startup, see your Linux distribution documentation on how
to configure persistem module loading.

"
    fi

    echo "#!/bin/sh" >>/etc/rc.modules
    echo "/sbin/modprobe $MODNAME" >>/etc/rc.modules
    chmod +x /etc/rc.modules
fi    
echo

# figure out where to put default module parameters (if needed)

if [ "$DEFOPTS" != "" ]; then
    if [ -d /etc/modprobe.d ]; then
        if [ -f /etc/modprobe.d/$MODNAME.conf ]; then
            echo "/etc/modprobe.d/$MODNAME.conf already exists"
        else
            echo "creating /etc/modprobe.d/$MODNAME.conf with default options:"
            echo "  $DEFOPTS"
            echo "options $MODNAME $DEFOPTS" > /etc/modprobe.d/$MODNAME.conf
        fi
        echo "

$MODNAME module parameters are in /etc/modprobe.d/$MODNAME.conf.  See
README.txt for information on setting port interface modes and for
polling and interrupt settings

"
    else
        echo "\

WARNING: Don't know where to put module parameters for this
         distribution.  If you need to change polling/interrupt
         settings or port interface modes, consult README.txt for
         module parameter names/values and consult your distribution
         documentation regarding where to set them.

"
    fi
fi